/*
Copyright (C) 2000 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "ckDialog.h"
#include "ckControl.h"
#include "ckScanner.h"
#include "ckStream.h"
#include "ckValueWrapper.h"
#include "ckControlTable.h"
#include "ckDialogControl.h"
#include "ckControlImp.h"
#include "ckVariables.h"
#include "ckMiscParser.h"


//
// ckControl
//
ckControl::ckControl(string &type)
:   _hwnd(NULL),
    _root(NULL)
{
    parent(NULL);
    addAttribute(TYPE_ATTRIB, type);
}


ckControl::~ckControl()
{
    _LSDEBUG1("~ckControl() dtor");
    
    if(getHWND())
    {
        // don't really want to put it here but oh heck
        // the beauty of recurse is just irrisistable
        _LSDEBUG1("~ckControl() sending DESTROY_EVENT");
        notify(CKDESTROY_EVENT);
        _LSDEBUG1("~ckControl() setting the window long to 0");
        SetWindowLong(getHWND(), GWL_USERDATA, 0);
    }
    
    parent(NULL);
    _hwnd = NULL;
}

//
// identity functions
//
void ckControl::addChild(auto_ptr<ckControl> &child)
{
    assert(child.get());
    child->parent(this);
    _children.push_back(child);
}


void ckControl::createwindow(DWORD xstyle, const char *classname, const char *windowname, DWORD style,
                    int x, int y, int w, int h, HWND parent, HMENU menu, HINSTANCE hinstance)
{   
    _hwnd = ::CreateWindowEx(xstyle, classname, windowname, style, x, y, w, h, parent, menu, hinstance, NULL);    
    
    if(!_hwnd) return;
    
    _LSDEBUG0;
    
    //these code should have some better place to be or not
    if(_attributeExists(FONTFACE_ATTRIB))
    {
        _LSDEBUG0;
        _font.reset(new Font(getAttribute(FONTFACE_ATTRIB).c_str()));
        if((HGDIOBJ)(*_font.get()))
            SendMessage(_hwnd, WM_SETFONT, (WPARAM)(HGDIOBJ)(*_font.get()), MAKELPARAM(FALSE, 0));
        else
            ckLogger::instance()->log(FAIL, "Error occured while loading font");
    
    }
    if(_attributeExists(FONTCOLOR_ATTRIB))
    {
        _LSDEBUG0;
        _LSDEBUG2("createwindow() from fontcolor: ", getAttribute(FONTCOLOR_ATTRIB).c_str());
        _LSDEBUG2("createwindow() to fontcolor: ", (DWORD)strtocolor(getAttribute(FONTCOLOR_ATTRIB).c_str()));

        _fontcolor.reset(new COLORREF);
        *(_fontcolor.get()) = strtocolor(getAttribute(FONTCOLOR_ATTRIB).c_str());
    }
    if(_attributeExists(BGCOLOR_ATTRIB))
    {
        _LSDEBUG0;
        _LSDEBUG2("createwindow() from bgcolor: ", getAttribute(BGCOLOR_ATTRIB).c_str());
        _LSDEBUG2("createwindow() to bgcolor: ", (DWORD)strtocolor(getAttribute(BGCOLOR_ATTRIB).c_str()));

        _bgcolor.reset(new COLORREF);
        *(_bgcolor.get()) = strtocolor(getAttribute(BGCOLOR_ATTRIB).c_str());
        _bgbrush.reset(new Brush(*(_bgcolor.get())));
    }
    _LSDEBUG0;
}
//
// child manipulation functions
//
ckError ckControl::removeChild(ckControl &control)
{
    auto_vector<ckControl>::iterator it = _children.find(&control);
    if(it != _children.end())
    {
        _LSDEBUG2("ckControlManager::remove() removing toplevel control with id: %s", it->getAttribute(ID_ATTRIB).c_str());
        _children.remove(it);
        return SUCCESS;
    }

    return FAIL;
}    

// 
// factory function
//
auto_ptr<ckControl> ckControl::Clone()
{
    auto_ptr<ckControl> specie = Create(getType());

    ckControl *ptr = specie.get();
    assert(ptr != NULL);
    assert(!getHWND());

    ptr->_attrib = _attrib;
    ptr->_event = _event;
    //ptr->parent(parent());    //the parent wouldn't work because of differnt address
                                //but don't worry because it'll be taken care of in the
                                //addChild mechanizm loop if we are inside recursion
    
    _LSDEBUG2("ckControl::Clone() cloned type %s", getType().c_str());
    _LSDEBUG2("ckControl::Clone() cloned %d attributes", (int)ptr->_attrib.size());
    _LSDEBUG2("ckControl::Clone() cloned %d events", (int)ptr->_event.size());
    //_LSDEBUG2("ckControl::Clone() cloned %d children", (int)ptr->_children.size());
    
    return specie;
}

//
// attributes functions
//
string ckControl::getAttribute(string const &attrib)
{   return _attributeExists(attrib) ? _attrib[attrib] : string();   }

ckError ckControl::setVariable(string const &var, string const &value)
{
    if(getHWND())
    {
        if( stricmp(var.c_str(), TYPE_ATTRIB)   == 0 ||
            stricmp(var.c_str(), ID_ATTRIB)     == 0 ||
            stricmp(var.c_str(), TITLE_ATTRIB)  == 0 ||
            stricmp(var.c_str(), SIZE_ATTRIB)   == 0 ||
            stricmp(var.c_str(), STYLE_ATTRIB)  == 0 ||
            stricmp(var.c_str(), XSTYLE_ATTRIB) == 0)
        {
            return INVALID_ACTION;
        }
    }
    
    addAttribute(var, value);
    return SUCCESS;
}

ckError ckControl::retreiveVariable(string const &var, string &value)
{ 
    value.erase();
    if(_attributeExists(var))
    {
        value = getAttribute(var);
        return SUCCESS;
    }

    return INVALID_ACTION;
}

// this function is for adding user defined attrib
ckError ckControl::addAttribute(string const &attrib, string const &value)
{
    /*
    if(!_attributeExists(attrib) && !hasAttribute(attrib))
        ckControlTable::instance()->addAttribute(attrib);
    */

    _attrib[attrib] = value;
    _LSDEBUG2("ckControl::addAttribute() added attribute:%s", attrib.c_str());
    _LSDEBUG2("ckControl::addAttribute() added value: %s", value.c_str());
    return SUCCESS;
}

ckError ckControl::addAttribute(string const &attrib, ckValueWrapper &wrapper)
{
    ckError error = INVALID_ATTRIBUTE;
    string str;

    _LSDEBUG1("WARNING: should check can't set default attribs after window creation");
    //storing for default attributes
    /*
    if(attrib == ID_ATTRIB)
    {
        error = wrapper.getStringWithLength(temp);
        
        if(error == SUCCESS)
        {
            addAttribute(attrib, temp);
            _LSDEBUG2("ckControl::addAttribute() got id %s", getAttribute(attrib).c_str());
        }
    }
    else if(attrib == TITLE_ATTRIB)
    {
        error = wrapper.getString(temp);
        if(error == SUCCESS)
        {
            addAttribute(attrib, temp);
            _LSDEBUG2("ckControl::addAttribute() got title %s", getAttribute(attrib).c_str());
        }
    }
    else */ //if(attrib == SIZE_ATTRIB)
    if(stricmp(attrib.c_str(),SIZE_ATTRIB) == 0)
    {
        // !! watch out that we are using right as width and bottom as height !!
        int num;
        string temp;

        error = wrapper.getVariableInt(num, temp);
        if(error == SUCCESS)
        {
            str = temp;
            error = wrapper.getVariableInt(num, temp);
            if(error == SUCCESS)
            {
                str += ' ' + temp;
                error = wrapper.getVariableInt(num, temp);
                if(error == SUCCESS)
                {
                    str += ' ' + temp;
                    error = wrapper.getVariableInt(num, temp);
                    if(error == SUCCESS)
                    {
                        str += ' ' + temp;
                        addAttribute(attrib, str);
                        _LSDEBUG2("ckControl::addAttribute() got size %d %d %d %d", getAttribute(attrib).c_str());
                    }
                }
            }
        }
    }
    else
    {
        //add everything else
        if(error != SUCCESS)
        {
            error = wrapper.getRawStringWithLength(str);
            if(error == SUCCESS)
                error = ckControl::addAttribute(attrib, str);
        }           
    }

    return error;
}

//
// event functions
//
ckError ckControl::addEvent(string const &event, string const &action)
{
    _LSDEBUGFUNS("ckControl::addEvent()");

    if(_event.find(event) == _event.end())
    {
        _event[event] = action;
        //_LSDEBUG2("ckControl::addEvent(() event %s with action \"%s\"", event.c_str(), action.c_str());
        _LSDEBUG2("ckControl::addEvent(() event \"%s\"", event.c_str());
        _LSDEBUG2("ckControl::addEvent(() action \"%s\"", , action.c_str());
        return SUCCESS;
    }
    
    _LSDEBUG1("ckControl::addEvent() failed");
    return FAIL;
}

ckError ckControl::notify(string const &event)
{
    _LSDEBUG0;
    ckControlItT it;
    _LSDEBUG0;
    it = _event.find(event);
    _LSDEBUG0;
    if(it != _event.end())
    {
        _LSDEBUG2("ckControl::notify(() on event %s", event.c_str());
        LSExecute(getHWND(), it->second.c_str(), 0);
        return SUCCESS;
    }
    _LSDEBUG0;
    
    return FAIL;
}

ckError ckControl::eventVariableSubstitution(string const &rootDialogID)
{
    string substitute("%[this]");
    for(ckControlItT it = _event.begin(); 
        it != _event.end(); 
        ++it)
    {
        int pos = 0;
        while(true)
        {
            pos = it->second.find(substitute, pos);
            if(pos != string::npos)
                it->second.replace(pos, substitute.size(), rootDialogID);
            else
                break;
        }
    }
    return SUCCESS;
}


DWORD ckControl::_parseStyle()
{
    DWORD style = WS_VISIBLE;

    _LSDEBUGFUNS("ckStaticControl::_parseStyle()");

    if(_getstyle(style))
    {
    _LSDEBUG1("ckControl::_parseStyle() parsing button style");
    ckStyleParser styleparser(style, _getstyle(style));
    styleparser.parse(ckStrStream(getAttribute(STYLE_ATTRIB)));
    }
    {
    _LSDEBUG1("ckControl::_parseStyle() parsing dialog style");
    //cool way for data inheritance?
    ckStyleParser styleparser(style, ckDialogControl::getstylegallery());
    styleparser.parse(ckStrStream(getAttribute(STYLE_ATTRIB)));
    }

    _LSDEBUG2("ckControl::_parseStyle() got button style: %08X", style);
    _LSDEBUG1("WARNING: should probably do a style check if want ot be safe");
    _LSDEBUG1("ckControl::_parseStyle() we should warn user if invalid style!");

    if(parentHWND())
    {
        _LSDEBUG1("ckControl::_parseStyle() auto set child");
        style |= WS_CHILD;
    }

    _LSDEBUG2("ckControl::_parseStyle() got total style: %08X", style);
    _LSDEBUGFUNE("ckControl::_parseStyle()");

    return style;
}

DWORD ckControl::_parseXStyle() 
{
    DWORD xstyle = 0;
    _LSDEBUGFUNS("ckControl::_parseXStyle()");
    ckStyleParser styleparser(xstyle, ckDialogControl::getxstylegallery());
    styleparser.parse(ckStrStream(getAttribute(XSTYLE_ATTRIB)));
    _LSDEBUG1("ckControl::_parseXStyle() we should warn user if invalid style!");
    _LSDEBUG2("ckControl::_parseXStyle() got xstyle: %08X", xstyle);
    _LSDEBUGFUNE("ckControl::_parseXStyle()");
    return xstyle;
}

bool ckControl::_attributeExists(string const &attrib)
{
    return (_attrib.find(attrib) != _attrib.end());
}

/*
auto_vector<ckControl>::iterator ckControl::_findChild(string const &id)
{
    auto_vector<ckControl>::iterator it;
    for(it = _children.begin(); it != _children.end(); ++it)
    {
        if(stricmp(it->getAttribute(ID_ATTRIB).c_str(), id.c_str()) == 0) break;
    }

    return it;
}
*/

//
// static factory method
//
auto_ptr<ckControl> ckControl::Create(string const &type)
{
    _LSDEBUG0;
    pMake ptr = ckControlTable::instance()->findControl(type);
    _LSDEBUG0;
    assert(ptr != NULL);
    _LSDEBUG2("ckControl::Create() creating %s", type.c_str());
    return ptr();
}

