/*
Copyright (C) 2000 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ckDialog.h"
#include "ckControlParser.h"
#include "ckScanner.h"
#include "ckStream.h"
#include "ckControlManager.h"
#include "ckVariables.h"

HINSTANCE g_hInstance = NULL;
   
//
// bang commands
//
void ckCreateDialog(HWND sender, LPCSTR args);
void ckAction(HWND sender, LPCSTR args);
void ckSetVariable(HWND sender, LPCSTR args);
void ckExecEvaluate(HWND sender, LPCSTR args);

void ckCreateDialog(HWND sender, LPCSTR args)
{
    _LSDEBUGFUNS("ckCreateDialog()");
    ckControlManager::instance()->createWindow(args);
    _LSDEBUGFUNE("ckCreateDialog()");
}

// ugg, dupe codes
void ckAction(HWND sender, LPCSTR args)
{
    _LSDEBUG2("!ckAction got argument", args);
    ckStrStream stream(args);
    ckDialogTemplateScanner scanner(stream);
    ckError error;

    // should remove
    if(scanner.accept() == ckTokenString)
    {
        error = ckControlManager::instance()->perform(scanner.getString(), stream.toString());
    }
    else
    {
        error = INVALID_VALUE;
        _LSDEBUG1("!ckAction invalid arguments");
    }
}

void ckSetVariable(HWND sender, LPCSTR args)
{
    _LSDEBUG2("!ckSetVariable got argument", args);
    ckStrStream stream(args);
    ckDialogTemplateScanner scanner(stream);
    ckError error;

    // should remove 
    if(scanner.accept() == ckTokenString)
    {
        error = ckControlManager::instance()->setVariable(scanner.getString(), stream.toString());
    }
    else
    {
        error = INVALID_VALUE;
        _LSDEBUG1("!ckSetVariable invalid arguments");
    }
}

void ckExecEvaluate(HWND sender, LPCSTR args)
{
    _LSDEBUG2("!ckExecEvaluate got argument", args);
    ckGlobalVariableReplacer varreplacer;
    string str = varreplacer.replace(ckStrStream(args));
    _LSDEBUG2("!ckExecEvaluate replaced as: ", str.c_str());
    LSExecute(sender, str.c_str(), 0);
}


//
// Litestep module specific functions
//
int initModule(HWND parent, HINSTANCE dll, wharfDataType* wd)
{
	return initModuleEx(parent, dll, (wd)? wd->lsPath : NULL);	
}

int initModuleEx(HWND parent, HINSTANCE dll, LPCSTR szPath)
{
    //
    // global init
    //
    g_hInstance = dll;
    ckControl::init(g_hInstance);

    try
    {
        ckError error;

        {
            _LSDEBUG0;
            ckDialogParser parser(*ckControlManager::instance());
            _LSDEBUG0;
            error = parser.parse(ckStepRCStream(CKDIALOGCONFIG));
            _LSDEBUG0;
            if(error != SUCCESS)
            {
                ckLogger::instance()->log(error, "Error encountered while parsing Dialog template");
            }
        }
        {
            ckEventHandlerParser eventparser(*ckControlManager::instance());
            _LSDEBUG0;
            error = eventparser.parse(ckStepRCStream(CKEVENTCONFIG));
            _LSDEBUG0;
            if(error != SUCCESS)
            {
                ckLogger::instance()->log(error, "Error occured while parsing the event handling config");
            }
        }

        
        _LSDEBUG0;

        //register bangs
        AddBangCommand("!ckCreateDialog", ckCreateDialog);        
        AddBangCommand("!ckAction", ckAction);
        AddBangCommand("!ckSetVariable", ckSetVariable);
        AddBangCommand("!ckExecEvaluate", ckExecEvaluate);
    }
    catch(...)
    {
        MessageBox(NULL, "unknown error", "ckDialog", MB_OK);
    }

    return 0;
}

int quitModule(HINSTANCE dll)
{
    RemoveBangCommand("!ckCreateDialog");
    RemoveBangCommand("!ckAction");
    RemoveBangCommand("!ckSetVariable");
    RemoveBangCommand("!ckExecEvaluate");
    return 0;
}
