/*
Copyright (C) 2000 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ckDialog.h"
#include "ckDialogControl.h"
#include "ckControlTable.h"
#include "ckControlImp.h"
#include "ckScanner.h"
#include "ckMiscParser.h"
#include "ckStream.h"
#include "ckVariables.h"
#include "ckValueWrapper.h"
//
// static 
//

ckControl::ControlStyleT ckDialogControl::_dialogstyle[] =
{
    {"wsvisible"             , WS_VISIBLE},
    {"wsminimize"	         , WS_MINIMIZE},
    {"wsmaximize"            , WS_MAXIMIZE},
    {"wsdisable"             , WS_DISABLED},
    {"wschild"               , WS_CHILD},
    {"wssizebox"             , WS_SIZEBOX},
    {"wsoverlapped"          , WS_OVERLAPPED},
    {"wsoverlappedwindow"    , WS_OVERLAPPEDWINDOW},

    {"wspopup"               , WS_POPUP},
    {"wspopupwindow"         , WS_POPUPWINDOW},

    {"wshscroll"             , WS_HSCROLL},
    {"wsvscroll"             , WS_VSCROLL},
        
    {"wsborder"              , WS_BORDER},
    {"wsdlgframe"			 , WS_DLGFRAME},
    {"wsthickborder"         , WS_THICKFRAME},
    {"wssysmenu"             , WS_SYSMENU},
    {"wscaption"             , WS_CAPTION},
    {"wsminimizebox"         , WS_MINIMIZEBOX},
    {"wsmaximizebox"         , WS_MAXIMIZEBOX},
    {"wsgroup"               , WS_GROUP},
    {"wstabstop"             , WS_TABSTOP},
    
    // unreleased
    {"wsclipchildren"        , WS_CLIPCHILDREN},
    {"wsclipsiblings"        , WS_CLIPSIBLINGS},

    {0, 0}
};


ckControl::ControlStyleT ckDialogControl::_dialogxstyle[] =
{
    {"wsxclientedge",       WS_EX_CLIENTEDGE},
    {"wsxtopmost",          WS_EX_TOPMOST},
    {"wsxtoolwindow",       WS_EX_TOOLWINDOW},
    {"wsxappwindow",        WS_EX_APPWINDOW},
    {"wsxacceptfiles",      WS_EX_ACCEPTFILES}, 
    //(going need to get the window message then parse it for them
    {"wsxmodalframe",       WS_EX_DLGMODALFRAME},

    // unsupported yet
    {"wsxacceptfiles",      WS_EX_ACCEPTFILES},
    {"wsxcontexthelp",      WS_EX_CONTEXTHELP},	
    {"wsxcontrolparent",    WS_EX_CONTROLPARENT},	
    {"wsxleft",             WS_EX_LEFT},
    {"wsxleftscroll",       WS_EX_LEFTSCROLLBAR},
    {"wsxltrreading",       WS_EX_LTRREADING},
    {"wsxmdichild",         WS_EX_MDICHILD},
    {"wsxnoparentnotify",   WS_EX_NOPARENTNOTIFY},
    {"wsxoverlappedwindow", WS_EX_OVERLAPPEDWINDOW},	
    {"wsxpalettewindow",    WS_EX_PALETTEWINDOW},
    {"wsxright",            WS_EX_RIGHT},
    {"wsxrightscroll",      WS_EX_RIGHTSCROLLBAR},
    {"wsxrtlreading",       WS_EX_RTLREADING},
    {"wsxstaticedge",       WS_EX_STATICEDGE},	
    {"wsxtransparent",      WS_EX_TRANSPARENT},
    {0, 0}
};

ckDialogControl::ckDialogControl()
:   ckControl(string(DIALOG_TYPE)),
    _icon(NULL) // must change!!!!!!!
{

}

void ckDialogControl::createwindow(DWORD xstyle, const char *classname, const char *windowname, DWORD style,
                int x, int y, int w, int h, HWND parent, HMENU menu, HINSTANCE hinstance)
{
    ckControl::createwindow(xstyle, classname, windowname, style,
                x, y, w, h, parent, menu, hinstance);
    if(getHWND())
    {
        if(_attributeExists(ICON_ATTRIB))
        {
            LoadIcon(getAttribute(ICON_ATTRIB));
        }
    }
}


ckError ckDialogControl::performAction(string const &action, ckValueWrapper &wrapper) 
{
    if(getHWND() != NULL)
    {
        _LSDEBUG2("ckDialogControl::performAction() action: %s", action.c_str());
        return ckControlImpMap<ckDialogImp>::instance()->execute(action, wrapper, (*this));
    }
    else
    {
        _LSDEBUG1("ckDialogControl::performAction() called when window hasn't been created");
        return FAIL;
    }
}


ckError ckDialogControl::retreiveVariable(string const &var, string &value)
{ 
    if(stricmp(var.c_str(), TITLE_ATTRIB) == 0)
    {
        //get the text of edit
        value = ckControlImpMap<ckDialogImp>::instance()->getImp()->_getText(*this);
    }
    else if(stricmp(var.c_str(), SIZE_ATTRIB) == 0)
    {
        RECT r;
        char buffer[0xFF];
        GetWindowRect(getHWND(), &r);
        
        sprintf(buffer, "%d %d %d %d", r.left, r.top, r.right-r.left, r.bottom-r.top);
        value = buffer;
    }
    else
    {
        return ckControl::retreiveVariable(var, value);
    }
    
    return SUCCESS;
}

/*
DWORD ckDialogControl::_parseStyle()
{
    DWORD style = WS_VISIBLE;

    _LSDEBUGFUNS("ckDialogControl::_parseStyle()");
    ckStyleParser styleparser(style, ckDialogControl::getstylegallery());
    styleparser.parse(ckStrStream(getAttribute(STYLE_ATTRIB)));
    _LSDEBUG1("ckDialogControl::_parseStyle() we should warn user if invalid style!");


    if(parentHWND())
    {
        _LSDEBUG1("ckDialogControl::_parseXStyle() auto set child");
        style |= WS_CHILD;
    }

    _LSDEBUG2("ckButtonControl::_parseStyle() got total style: %08X", style);
    _LSDEBUGFUNE("ckDialogControl::_parseStyle()");
    return style;
}
*/

void ckDialogControl::LoadIcon(string const &filename)
{
    if(_icon)
        DeleteObject((HGDIOBJ)_icon);

    _icon = (HICON)::LoadImage(NULL, filename.c_str(), IMAGE_ICON,
						0, 0, LR_DEFAULTCOLOR|LR_LOADFROMFILE);
    if(_icon)
    {
        ::SendMessage(getHWND(), WM_SETICON, (WPARAM)ICON_SMALL, (LPARAM)_icon);
    }
    else
    {
        ckLogger::instance()->log(FAIL, "Failed to load icon");
    }
}