/*
Copyright (C) 2000 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ckDialog.h"
#include "ckDialogControl.h"
#include "ckStaticControl.h"
#include "ckControlTable.h"
#include "ckControlImp.h"
#include "ckScanner.h"
#include "ckMiscParser.h"
#include "ckStream.h"
#include "ckVariables.h"
#include "ckValueWrapper.h"
#include "ckPainter.h"

//
// static 
//
ckControl::ControlStyleT ckStaticControl::_staticstyle[] =
{
    {"ssblackframe"     ,   SS_BLACKFRAME},
    {"ssblackrect"      ,   SS_BLACKRECT},
    {"sscenter"         ,   SS_CENTER},
    {"sscenterimage"    ,   SS_CENTERIMAGE},
    {"ssgrayframe"      ,   SS_GRAYFRAME},
    {"ssgrayrect"       ,   SS_GRAYRECT},
    {"ssleft"           ,   SS_LEFT},
    {"ssleftnowordwrap" ,   SS_LEFTNOWORDWRAP},
    {"ssnoprefix"       ,   SS_NOPREFIX},
    {"ssright"          ,   SS_RIGHT},
    {"sssimple"         ,   SS_SIMPLE},
    {"sswhiteframe"     ,   SS_WHITEFRAME},
    {"sswhiterect"      ,   SS_WHITERECT},
    {"ssnotify"         ,   SS_NOTIFY},

    {"ssbitmap"         ,   SS_BITMAP},
    {"ssicon"           ,   SS_ICON},
    {0, 0}
};



ckStaticControl::ckStaticControl()
:   ckControl(string(STATIC_TYPE))
{

}


void ckStaticControl::createwindow(DWORD xstyle, const char *classname, const char *windowname, DWORD style,
                int x, int y, int w, int h, HWND parent, HMENU menu, HINSTANCE hinstance)
{
    ckControl::createwindow(xstyle, classname, windowname, style,
                x, y, w, h, parent, menu, hinstance);
    if(getHWND())
    {
        if(_attributeExists(BITMAP_ATTRIB))
            LoadBitmap(getAttribute(BITMAP_ATTRIB));
        
    }
}

ckError ckStaticControl::performAction(string const &action, ckValueWrapper &wrapper) 
{
    if(getHWND() != NULL)
    {
        _LSDEBUG2("ckButtonControl::performAction() action: %s", action.c_str());
        return ckControlImpMap<ckStaticImp>::instance()->execute(action, wrapper, (*this));
    }
    else
    {
        _LSDEBUG1("ckButtonControl::performAction() called when window hasn't been created");
        return FAIL;
    }
}

ckError ckStaticControl::retreiveVariable(string const &var, string &value)
{ 
    if(stricmp(var.c_str(), TITLE_ATTRIB) == 0)
    {
        //get the text of edit
        value = ckControlImpMap<ckStaticImp>::instance()->getImp()->_getText(*this);
    }
    else
    {
        return ckControl::retreiveVariable(var, value);
    }
    
    return SUCCESS;
}

/*
DWORD ckStaticControl::_parseStyle()
{
    DWORD style = WS_VISIBLE;

    _LSDEBUGFUNS("ckStaticControl::_parseStyle()");

    {
    _LSDEBUG1("ckStaticControl::_parseStyle() parsing button style");
    ckStyleParser styleparser(style, ckStaticControl::getstylegallery());
    styleparser.parse(ckStrStream(getAttribute(STYLE_ATTRIB)));
    }
    {
    _LSDEBUG1("ckStaticControl::_parseStyle() parsing dialog style");
    //cool way for data inheritance?
    ckStyleParser styleparser(style, ckDialogControl::getstylegallery());
    styleparser.parse(ckStrStream(getAttribute(STYLE_ATTRIB)));
    }

    _LSDEBUG2("ckStaticControl::_parseStyle() got button style: %08X", style);
    _LSDEBUG1("WARNING: should probably do a style check if want ot be safe");
    _LSDEBUG1("ckStaticControl::_parseStyle() we should warn user if invalid style!");
    
    if(parentHWND())
    {
        _LSDEBUG1("ckStaticControl::_parseStyle() auto set child");
        style |= WS_CHILD;
    }

    _LSDEBUG2("ckStaticControl::_parseStyle() got total style: %08X", style);
    _LSDEBUGFUNE("ckStaticControl::_parseStyle()");

    return style;
}
*/

void ckStaticControl::LoadBitmap(string const &filename)
{
    assert(getHWND());
    if(_bitmap.LoadFile(filename.c_str()))
    {
        SendMessage(getHWND(), STM_SETIMAGE, (WPARAM)IMAGE_BITMAP, (LPARAM)(HBITMAP)_bitmap);
    }
    else
    {
        ckLogger::instance()->log(FAIL, "Failed to load bitmap");
    }
}