/*
Copyright (C) 2001 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ckDialog.h"
#include "ckStream.h"

//
// ckStream
//
string ckStream::toString()
{
    string str;
    while(isMore())
    {
        str += next();
    }

    return str;
}


//
// ckPositionMemento
//
ckPositionMemento::ckPositionMemento(ckPositionableStream &stream) 
:   _stream(stream)
{
    _pos = _stream.getPos();
}

ckPositionMemento::~ckPositionMemento()
{
    _stream.setPos(_pos);
}

//
// ckStrStream
//
ckStrStream::ckStrStream(string const &src)
:   _src(src)
{
    _index = 0;
    _length = _src.length();
    //_LSDEBUG2("ckStrStream() ctor with string: %s length: %d index: %d", _src.c_str(), _length, _index);
    _LSDEBUG2("ckStrStream() ctor with string: %s length: %d index: %d", _src.c_str());
}

bool ckStrStream::isMore()
{
    return _index < _length;
}

char ckStrStream::next()
{
    assert(isMore());
    return _src[_index++];
}

char ckStrStream::peek()
{
    assert((_index + 1) < _length);
    return _src[_index+1];
}

void ckStrStream::setPos(int pos)
{
    _index = pos;
}

int ckStrStream::getPos()
{
    return _index;
}

void ckStrStream::reset()
{
    _index = 0;
}

// our own 
string ckStrStream::toString()
{
    if(isMore())
    {
        return _src.substr(_index);
    }
    
    return string();
}


//
// ckStepRCStream
//
ckStepRCStream::ckStepRCStream(string const &lsconfig) 
:   _lsconfig(lsconfig)
{
    _isMore = true; //getMore will only set to false
    _file = LCOpen(NULL);
    _getMore();
}

ckStepRCStream::~ckStepRCStream() 
{
    if(_file != NULL)
    {
        LCClose(_file);
    }
}

bool ckStepRCStream::isMore()
{
    assert(_file);
    _getMore();
    return _isMore;
}

char ckStepRCStream::next()
{
    assert(_file);
    assert(_isMore && _strstream->isMore());
    //want next? must be originally isMore()
    return _strstream->next();
}
 

void ckStepRCStream::_getMore()
{
    // only read next if currently no more
    if(_isMore && (!_strstream.get() || !_strstream->isMore()))
    {
        char buffer[MAX_LINE_LENGTH];
        char expand[MAX_LINE_LENGTH];
        char *tokenprobe = NULL;

        /* only gets one non empty config line */
        while(LCReadNextConfig(_file, _lsconfig.c_str(), buffer, MAX_LINE_LENGTH))
        {
            _LSDEBUG2("LCReadNextConfig() got line: %s", buffer);
            VarExpansion(expand, buffer);
            _LSDEBUG2("expanded: %s", expand);
            tokenprobe = expand;

            /* strip off the *config */
            while(*tokenprobe && !isspace(*tokenprobe++))
                continue;
                            
            while(*tokenprobe && isspace(*tokenprobe))
                ++tokenprobe;

            if(*tokenprobe)
            {
                _LSDEBUG2("probed: %s", tokenprobe);
                _strstream.reset(new ckStrStream(string("\n")+tokenprobe));
                break;
            }

            tokenprobe = NULL; //reset
        }

        if(tokenprobe == NULL)
        {
            _isMore = false;
        }
    }
}
