#if !defined (AUTOPTR_H)
#define AUTOPTR_H
//------------------------------------
// (c) Bartosz Milewski 1998
//------------------------------------

//-------------
// Strong pointer
//-------------
template<class T>
class smart_ptr
{
	template<class U> struct smart_ptr_ref
	{
		smart_ptr<U> & _ap;
		smart_ptr_ref (smart_ptr<U> & p) : _ap (p) {}
	};
public:
	typedef T element_type;

	explicit smart_ptr (T * p = 0) : _p (p) {}

	smart_ptr (smart_ptr<T> & pSrc)
	{
		_p = pSrc._p; // implicit cast
		pSrc._p = 0;
	}
	
	smart_ptr & operator= (smart_ptr<T> & pSrc)
	{
		if (this != &pSrc)
		{
            reset(NULL);
			_p = pSrc._p;
			pSrc._p = 0;
		}
		return (*this);
	}

	~smart_ptr () 
    { 
        reset(NULL);
    }

	T & operator* () const { return *_p; }

	T * get () const { return _p; }

	T * release ()
	{
		T * tmp = _p;
		_p = 0;
		return tmp;
	}

	void reset (T * p = 0)
	{
	    if (_p != NULL && p != _p)
	    {
		    delete _p;
		    _p = p;
	    }
	}

	smart_ptr (smart_ptr_ref<T> apr)
		: _p (apr._ap.release ())
	{}

	template<class U>
	operator smart_ptr_ref<U> ()
	{
		return smart_ptr_ref<U> (*this);
	}

	template<class U>
	operator smart_ptr<U> ()
	{
		return smart_ptr<U> (release ());
	}
private:
	T * _p;
};

//-----------
// Auto array
//-----------
template<class T>
class smart_array
{
	template<class U> struct smart_array_ref
	{
		smart_array<U> & _aa;
		smart_array_ref (smart_array<U> & aa) : _aa (aa) {}
	};
public:
	typedef T element_type;

	explicit smart_array (size_t size)
		: _a (size != 0? new T [size]: 0) 
	{}

	explicit smart_array (T * p = 0) : _a (p) {}

	// "transfer" constructor
	smart_array (smart_array & aSrc)
	{
		_a = aSrc._a;
		aSrc._a = 0;
	}

	smart_array & operator= (smart_array & aSrc)
	{
		if (this != &aSrc)
		{
			reset(NULL);
			_a = aSrc._a;
			aSrc._a = 0;
		}
		return *this;
	}
#if 0 // not in this version of the compiler
	// "transfer" from derived class
	template<class U> 
	smart_array (smart_array<U> & aSrc)
	{
		_a = aSrc._a; // implicit cast
		aSrc._a = 0;
	}

	// assignment of derived class
	template<class U> 
	smart_array & operator= (smart_array<U> & aSrc)
	{
		if (this != &aSrc)
		{
			reset(NULL);
			_a = aSrc._a;
			aSrc._a = 0;
		}
		return *this;
	}
#endif
	~smart_array () { reset(NULL); }

	T const & operator [] (int i) const { return _a [i]; }

	T & operator [] (int i) { return _a [i]; }

	T * get () const { return _a; }

	T * release ()
	{
		T * tmp = _a;
		_a = 0;
		return tmp;
	}

	void reset (T * p = 0)
	{
		if (_a != NULL && p != _a)
		{
			delete []_a;
			_a = p;
		}
	}

	smart_array (smart_array_ref<T> apr)
		: _a (apr._ap.release ())
	{}

	template<class U>
	operator smart_array_ref<U> ()
	{
		return smart_array_ref<U> (*this);
	}

	template<class U>
	operator smart_array<U> ()
	{
		return smart_array<U> (release ());
	}
private:
	T * _a;
};



#endif
